package be.jeffcheasey88.peeratcode.routes;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;

import org.json.simple.JSONObject;

import be.jeffcheasey88.peeratcode.model.Completion;
import be.jeffcheasey88.peeratcode.model.Player;
import be.jeffcheasey88.peeratcode.model.Puzzle;
import be.jeffcheasey88.peeratcode.repository.DatabaseRepository;
import be.jeffcheasey88.peeratcode.webserver.HttpReader;
import be.jeffcheasey88.peeratcode.webserver.HttpUtil;
import be.jeffcheasey88.peeratcode.webserver.HttpWriter;
import be.jeffcheasey88.peeratcode.webserver.Response;
import be.jeffcheasey88.peeratcode.webserver.Route;
import be.jeffcheasey88.peeratcode.webserver.User;

public class PuzzleResponse implements Response {
	private final DatabaseRepository databaseRepo;
	private final String usersFilesPath;

	public PuzzleResponse(DatabaseRepository databaseRepo, String initUsersFilesPath) {
		this.databaseRepo = databaseRepo;
		usersFilesPath = initUsersFilesPath;
	}

	@Route(path = "^\\/puzzleResponse\\/([0-9]+)$", type = "POST")
	@Override
	public void exec(Matcher matcher, User user, HttpReader reader, HttpWriter writer) throws Exception {
		if (user == null) {
			HttpUtil.responseHeaders(writer, 401, "Access-Control-Allow-Origin: *");
			return;
		}

		ReceivedResponse received = new ReceivedResponse(matcher, reader);
		saveSourceCode(received, databaseRepo.getPlayer(user.getId()));

		JSONObject responseJSON = new JSONObject();
		Puzzle currentPuzzle = databaseRepo.getPuzzle(received.getPuzzleId());
		Completion completion = databaseRepo.insertOrUpdatePuzzleResponse(received.getPuzzleId(), user.getId(),
				received.getFileName(), received.getSourceCode(), received.getResponse(), currentPuzzle);
		if (completion.getScore() > 0) {
			HttpUtil.responseHeaders(writer, 200, "Access-Control-Allow-Origin: *", "Content-Type: application/json");
			responseJSON.put("score", completion.getScore());
			responseJSON.put("tries", completion.getTries());

		} else if (completion != null) {
			HttpUtil.responseHeaders(writer, 406, "Access-Control-Allow-Origin: *", "Content-Type: application/json");
			responseJSON.put("tries", completion.getTries());
		} else {
			HttpUtil.responseHeaders(writer, 400, "Access-Control-Allow-Origin: *");
			return;
		}
		writer.write(responseJSON.toJSONString());
	}

	private void saveSourceCode(ReceivedResponse received, Player player) {
		try {
			Path path = Paths.get(String.format("%s/%s/puz%04d-%s", usersFilesPath, player.getPseudo(),
					received.getPuzzleId(), received.getFileName()));
			Files.write(path, received.getSourceCode());
		} catch (IOException e) {
			System.err.println("ERREUR D'ENREGISTREMENT DU CODE POUR PUZZLE_RESPONSE : " + e.toString());
		}

	}

}

class ReceivedResponse {

	private int puzzleId;
	private byte[] response;
	private String fileName;
	private byte[] sourceCode;

	public ReceivedResponse(Matcher matcher, HttpReader reader) throws Exception {
		puzzleId = Integer.parseInt(matcher.group(1));

		List<String> multiPartData = HttpUtil.readMultiPartData(reader);
		this.response = multiPartData.get(0).getBytes();
		this.fileName = multiPartData.get(1);
		this.sourceCode = multiPartData.get(2).getBytes();
	}

	public int getPuzzleId() {
		return puzzleId;
	}

	public byte[] getResponse() {
		return response;
	}

	public String getFileName() {
		return fileName;
	}

	public byte[] getSourceCode() {
		return sourceCode;
	}
}
